<?php

namespace DMG\Newsletters;

/**
 * Class API
 *
 * REST endpoint actions to support editor.
 *
 * @package DMG\Newsletters
 */
class API {

	/**
	 * Meta key to store the Sailthru list.
	 * 
	 * @var string
	 */
	const META_KEY_SAILTHRU_LIST = 'dmg_newsletter_sailthru_list';

	/**
	 * Meta key to store frequency value.
	 * 
	 * @var string
	 */
	const META_KEY_FREQUENCY = 'dmg_newsletter_frequency';

	/**
	 * Meta key to store MSN text.
	 * 
	 * @var string
	 */
	const META_KEY_MSN_TEXT = 'dmg_newsletter_msn_text';

	/**
	 * Meta key to indicate if this newsletter has a postcode field.
	 * 
	 * @var string
	 */
	const META_KEY_HAS_POSTCODE = 'dmg_newsletter_has_postcode';

	/**
	 * Meta key to indicate if this newsletter has a date of birth field.
	 * 
	 * @var string
	 */
	const META_KEY_HAS_BIRTH_DATE = 'dmg_newsletter_has_birth_date';

	/**
	 * Meta key to indicate if this should be visible on the homepage.
	 * 
	 * @var string
	 */
	const META_KEY_VISIBLE_HOMEPAGE = 'dmg_newsletter_visible_homepage';

	/**
	 * Meta key to indicate if this should be visible on the category page.
	 * 
	 * @var string
	 */
	const META_KEY_VISIBLE_CATEGORY = 'dmg_newsletter_visible_category';

	/**
	 * Meta key to indicate if this should be visible on the tag page.
	 * 
	 * @var string
	 */
	const META_KEY_VISIBLE_TAG = 'dmg_newsletter_visible_tag';

	/**
	 * Init.
	 * 
	 * @return void
	 */
	public static function init(): void {
		add_action( 'rest_api_init', array( __CLASS__, 'register_rest_fields' ) );
	}

	/**
	 * Register REST fields.
	 * 
	 * @return void
	 */
	public static function register_rest_fields(): void {

		// Register the Sailthru list field.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_SAILTHRU_LIST,
			array(
				'get_callback'    => fn( array $object ) => get_post_meta( $object['id'], self::META_KEY_SAILTHRU_LIST, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_SAILTHRU_LIST, sanitize_text_field( $value ) ),
			)
		);

		// Register the frequency field.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_FREQUENCY,
			array(
				'get_callback'    => fn( array $object ) => get_post_meta( $object['id'], self::META_KEY_FREQUENCY, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_FREQUENCY, sanitize_text_field( $value ) ),
			)
		);

		// Register the MSN text field.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_MSN_TEXT,
			array(
				'get_callback'    => fn( array $object ) => get_post_meta( $object['id'], self::META_KEY_MSN_TEXT, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_MSN_TEXT, sanitize_text_field( $value ) ),
			)
		);

		// Register the postcode toggle.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_HAS_POSTCODE,
			array(
				'get_callback'    => fn( array $object ) => (bool) get_post_meta( $object['id'], self::META_KEY_HAS_POSTCODE, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_HAS_POSTCODE, (bool) $value ),
			)
		);

		// Register the date of birth toggle.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_HAS_BIRTH_DATE,
			array(
				'get_callback'    => fn( array $object ) => (bool) get_post_meta( $object['id'], self::META_KEY_HAS_BIRTH_DATE, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_HAS_BIRTH_DATE, (bool) $value ),
			)
		);

		// Register the homepage visibility toggle.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_VISIBLE_HOMEPAGE,
			array(
				'get_callback'    => fn( array $object ) => (bool) get_post_meta( $object['id'], self::META_KEY_VISIBLE_HOMEPAGE, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_VISIBLE_HOMEPAGE, (bool) $value ),
			)
		);

		// Register the category visibility toggle.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_VISIBLE_CATEGORY,
			array(
				'get_callback'    => fn( array $object ) => (bool) get_post_meta( $object['id'], self::META_KEY_VISIBLE_CATEGORY, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_VISIBLE_CATEGORY, (bool) $value ),
			)
		);

		// Register the tag page visibility toggle.
		register_rest_field(
			Post_Type::NAME,
			self::META_KEY_VISIBLE_TAG,
			array(
				'get_callback'    => fn( array $object ) => (bool) get_post_meta( $object['id'], self::META_KEY_VISIBLE_TAG, true ),
				'update_callback' => fn( string $value, \WP_Post $object ) => update_post_meta( $object->ID, self::META_KEY_VISIBLE_TAG, (bool) $value ),
			)
		);
	}
}

API::init();
